@@ -1,7 +1,7 @@
 /*
  * This file is part of sidplayfp, a console SID player.
  *
- * Copyright 2011-2022 Leandro Nini
+ * Copyright 2011-2023 Leandro Nini
  * Copyright 2000-2001 Simon White
  *
  * This program is free software; you can redistribute it and/or modify
@@ -52,6 +52,16 @@ using std::endl;
 #include <sidplayfp/SidInfo.h>
 #include <sidplayfp/SidTuneInfo.h>
 
+#ifdef HAVE_CXX11
+#  include <unordered_map>
+    typedef std::unordered_map<std::string, double> filter_map_t;
+    typedef std::unordered_map<std::string, double>::const_iterator filter_map_iter_t;
+#else
+#  include <map>
+    typedef std::map<std::string, double> filter_map_t;
+    typedef std::map<std::string, double>::const_iterator filter_map_iter_t;
+#endif
+
 // Previous song select timeout (4 secs)
 #define SID2_PREV_SONG_TIMEOUT 4000
 
@@ -103,6 +113,70 @@ uint16_t freqTableNtsc[]
 };
 #endif
 
+// This table contains chip-profiles which allow us to adjust
+// certain settings that varied wildly between 6581 chips, even
+// made in the same factory on the same day.
+//
+// This works under the assumption that the authors used the
+// same SID chip their entire career.
+//
+// based on https://github.com/reFX/libSidplayEZ/blob/modernized/src/chip-profiles.h
+static const filter_map_t filterCurveMap =
+{
+    { "Anthony Lees",                        0.450 },
+    { "Antony Crowther (Ratt)",              0.400 },
+    { "Ben Daglish",                         0.900 },
+    { "Charles Deenen",                      1.000 },
+    { "Chris H\xFClsbeck",                   0.600 },
+    { "David Dunn",                          1.100 },
+    { "David Dunn & Aidan Bell",             1.100 },
+    { "David Whittaker",                     1.000 },
+    { "Thomas Mogensen (DRAX)",              0.450 },
+    { "Edwin van Santen",                    0.650 },
+    { "Falco Paul",                          1.100 },
+    { "Figge Wasberger (Fegolhuzz)",         1.100 },
+    { "Fred Gray",                           1.250 },
+    { "Geir Tjelta",                         0.700 },
+    { "Georg Feil",                          1.250 },
+    { "Glenn Rune Gallefoss",                1.250 },
+    { "Graham Jarvis & Rob Hartshorne",      1.000 },
+    { "Jason Page",                          1.000 },
+    { "Jeroen Tel",                          0.825 },
+    { "Johannes Bjerregaard",                0.700 },
+    { "Jonathan Dunn",                       0.750 },
+    { "Jouni Ikonen (Mixer)",                0.600 },
+    { "Kim Christensen (Future Freak)",      0.300 },
+    { "Linus \xC5kesson (lft)",              0.900 },
+    { "Mark Cooksey",                        1.250 },
+    { "Markus M\xFCller (Superbrain)",       0.800 },
+    { "Martin Walker",                       1.000 },
+    { "Matt Gray",                           1.100 },
+    { "Michael Hendriks",                    0.900 },
+    { "M. Nilsson-Vonderburgh (Mic)",        0.700 },
+    { "M. Nilsson-Vonderburgh (Mitch)",      0.700 },
+    { "M. Nilsson-Vonderburgh (Yankee)",     0.700 },
+    { "Neil Brennan",                        0.750 },
+    { "Peter Clarke",                        0.600 },
+    { "Pex Tufvesson (Mahoney)",             0.400 },
+    { "Pex Tufvesson (Zax)",                 0.400 },
+    { "Renato Brosowski (Zoci-Joe)",         1.400 },
+    { "Reyn Ouwehand",                       1.000 },
+    { "Richard Joseph",                      0.700 },
+    { "Rob Hubbard",                         0.700 },
+    { "Russell Lieblich",                    0.400 },
+    { "Stellan Andersson (Dane)",            0.350 },
+    { "Steve Turner",                        0.550 },
+    { "Tim Follin",                          0.300 },
+    { "Thomas E. Petersen (Laxity)",         1.550 },
+    { "Thomas E. Petersen (TSS)",            1.550 },
+};
+
+double getRecommendedFilterCurve(const std::string& author)
+{
+    filter_map_iter_t it = filterCurveMap.find(author);
+    return (it != filterCurveMap.end()) ? it->second : 0.5;
+}
+
 uint8_t* loadRom(const SID_STRING &romPath, const int size)
 {
     SID_IFSTREAM is(romPath.c_str(), std::ios::binary);
@@ -178,9 +252,11 @@ ConsolePlayer::ConsolePlayer (const char * const name) :
     m_state(playerStopped),
     m_outfile(NULL),
     m_filename(""),
+    m_fcurve(-1.0),
     m_quietLevel(0),
     m_cpudebug(false),
-    newSonglengthDB(false)
+    newSonglengthDB(false),
+    m_autofilter(false)
 {
 #ifdef FEAT_REGS_DUMP_SID
     memset(m_registers, 0, 32*3);
@@ -263,7 +339,7 @@ ConsolePlayer::ConsolePlayer (const char * const name) :
     m_verboseLevel = (m_iniCfg.sidplay2()).verboseLevel;
 
     createOutput (OUT_NULL, nullptr);
-    createSidEmu (EMU_NONE);
+    createSidEmu (EMU_NONE, nullptr);
 
     uint8_t *kernalRom = loadRom((m_iniCfg.sidplay2()).kernalRom, 8192, TEXT("kernal"));
     uint8_t *basicRom = loadRom((m_iniCfg.sidplay2()).basicRom, 8192, TEXT("basic"));
@@ -422,7 +498,7 @@ bool ConsolePlayer::createOutput (OUTPUTS driver, const SidTuneInfo *tuneInfo)
 
 
 // Create the sid emulation
-bool ConsolePlayer::createSidEmu (SIDEMUS emu)
+bool ConsolePlayer::createSidEmu (SIDEMUS emu, const SidTuneInfo *tuneInfo)
 {
     // Remove old driver and emulation
     if (m_engCfg.sidEmulation)
@@ -448,10 +524,45 @@ bool ConsolePlayer::createSidEmu (SIDEMUS emu)
             rs->create ((m_engine.info ()).maxsids());
             if (!rs->getStatus()) goto createSidEmu_error;
 
-            if (m_filter.filterCurve6581)
-                rs->filter6581Curve(m_filter.filterCurve6581);
-            if (m_filter.filterCurve8580)
-                rs->filter8580Curve((double)m_filter.filterCurve8580);
+            double fcurve;
+            if (m_autofilter && (tuneInfo->numberOfInfoStrings() == 3))
+            {
+                fcurve = getRecommendedFilterCurve(tuneInfo->infoString(1));
+                if (m_verboseLevel > 1)
+                    cerr << "Recommended filter curve: " << fcurve << endl;
+            }
+            else if (m_fcurve >= 0.0)
+            {
+                fcurve = m_fcurve;
+            }
+            else if (m_filter.filterCurve6581 >= 0.0)
+            {
+                fcurve = m_filter.filterCurve6581;
+            }
+
+            if (fcurve >= 0.0)
+            {
+                if (m_verboseLevel)
+                    cerr << "6581 filter curve: " << fcurve << endl;
+                rs->filter6581Curve(fcurve);
+            }
+
+            fcurve = -1.0;
+            if (m_fcurve >= 0.0)
+            {
+                fcurve = m_fcurve;
+            }
+            if (m_filter.filterCurve8580 >= 0.0)
+            {
+                fcurve = m_filter.filterCurve8580;
+            }
+
+            if (fcurve >= 0.0)
+            {
+                if (m_verboseLevel)
+                    cerr << "8580 filter curve: " << fcurve << endl;
+                rs->filter8580Curve(fcurve);
+            }
         }
         catch (std::bad_alloc const &ba) {}
         break;
@@ -567,7 +678,7 @@ bool ConsolePlayer::open (void)
         m_track.songs = tuneInfo->songs();
     if (!createOutput(m_driver.output, tuneInfo))
         return false;
-    if (!createSidEmu(m_driver.sid))
+    if (!createSidEmu(m_driver.sid, tuneInfo))
         return false;
 
     // Configure engine with settings
@@ -649,7 +760,7 @@ void ConsolePlayer::close ()
 
     // Shutdown drivers, etc
     createOutput    (OUT_NULL, nullptr);
-    createSidEmu    (EMU_NONE);
+    createSidEmu    (EMU_NONE, nullptr);
     m_engine.load   (nullptr);
     m_engine.config (m_engCfg);
 
